; Copyright (c) 2004-2010, Laurent Jorda
;   All rights reserves
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; NAME:
;       resample_nac_image
;
; PROJECT:
;       OSIRIS
;
; PURPOSE:
;       This routine allows to resample an image 'iima' by interpolating the
;       value of the pixels at coordinates defined by the 'dpix' variable.
;       The output resampled image is stored in the 'oima' variable. This
;       routine calls several IDL routines which perform interpolation using
;       several methods.
;
; CALLING SEQUENCE:
;       resample_nac_image, xarr, yarr, iima, oima, $
;                          METHOD=method, IFAIL=ifail, DEBUG=debug
;
; INPUTS:
;       xarr
;         Array of dimension (NX,NY), where NX is the dimension of the
;         acquired image along X and NY its Y dimension, which contains the
;         X coordinates in the RCCD frame (starting at X=1) of the pixels of
;         the de-distorted image.
;
;       yarr
;         Array of dimension (NX,NY), where NX is the dimension of the
;         acquired image along X and NY its Y dimension, which contains the
;         Y coordinates in the RCCD frame (starting at Y=1) of the pixels of
;         the de-distorted image.
;
;       iima
;         The input image (2D float 32 bits array) to be resampled.
;
; OUTPUTS:
;       oima
;         The output resampled image (2D float 32 bits array).
;
; KEYWORD PARAMETERS:
;       method
;         The method to be used in a character string:
;
;           "BL" -> bilinear interpolation (IDL routine BILINEAR)
;
;       debug
;         Set to select debug mode. Optional.
;
;       ifail
;         Return the integer error code: 0=OK, >0=error occured
;
; DEPENDENCIES:
;
; REMARKS:
;
; MODIFICATION HISTORY:
;       2006/11/29: written by L. Jorda (LAM)
;------------------------------------------------------------------------------
PRO resample_nac_image, xarr, yarr, iima, oima, $
                       METHOD=method, IFAIL=ifail, DEBUG=debug

;
; START OF ROUTINE
; ----------------
; initialization
;
debug = keyword_set(debug)

if (debug) then print,'M: RESAMPLE_IMAGE_02 - BEGIN'

;
; default method code and maximum degree of polynomial
;
if (n_elements(method) eq 0)  then method  = 'BL'   ; use bilinear interpolation

if (debug) then print,'M:Code for method is [',method,'].'

;
; size of input image 'iima'
;
s  = size(iima)
nx = fix(s(1))
ny = fix(s(2))

if (debug) then print,'M:Size of input image: ',nx,'x',ny,' pixels.'

;
; creating dummy output image
;
oima = fltarr(nx,ny)

;
; method "BL" -> bilinear interpolation (IDL routine BILINEAR)
; ------------------------------------------------------------
if (method eq 'BL') then begin

    if (debug) then print,'M: -- Resampling routine: BILINEAR // IDL.'

;
; calling interpolation routine, this creates image 'oima'
;
    if (debug) then print,'M: -- Calling BILINEAR resampling routine.'

    oima  = BILINEAR(iima, xarr-1.0, yarr-1.0, MISSING=0.0)
    ifail = 0

endif

;
; check for internal errors during resampling
;
if (debug) then print,'M: -- Error code from routine: ',ifail,'.'

if (ifail ne 0) then begin

    if (debug) then print,'E:Error calling interpolation routine!'

    ifail = 10
    return

endif

;
; normal end
;
if (debug) then print,'M: RESAMPLE_IMAGE_02 - END'

; fin
RETURN
END
