; Copyright (c) 2004-2010, Stubbe F. Hviid
;   All rights reserved
;+
; NAME:
;   FUNCTION osiris_exposuretime, header, image
;
; PURPOSE:
;   Convert an Osiris image from DN to DN/s taking binning and shutter motion into account
;
; CATEGORY:
;   Osiris CCD Calibration
;
; CALLING SEQUENCE:
;  Result = osiris_exposuretime(header, image, quality_map, sigma_map)
;
; INPUTS:
;   header:   	The PDS image header of the image to be corrected
;   image:      	The image data of the image to be corrected
;   quality_map:	Existing map of quality estimate for each pixel to be modified
;   sigma_map:		Existing map of estimaped relative error for each pixel (in %) to be modified
;
; KEYWORD PARAMETERS:
;   PARAMETER_FILELIST: all external datafile names neede for this correction step
;                  will be added to this keywor parameter in a string array
;
; OUTPUTS:
;   header:   		The PDS image header of the image after correction
;   image:      	The image data of the image after correction
;   quality_map:	output map of quality estimate for each pixel
;   sigma_map:		output map of estimaped relative error for each pixel (in %)
;
; RETURNS:
;   0 on failure   1 on success
;
; RESTRICTIONS:
;   function requires that the paths defined in get_osiris_environment.pro has been set to correctly
;   point to the root directory of the external calibration files
;
; EXAMPLE:
;   p_read, pickfile(), h, d             ; read in raw PDS image
;   osiris_exposuretime, h, d
;
; MODIFICATION HISTORY:
;   Written by:  Stubbe F. Hviid   28/6-2005
;                Approximate correction between commanded and actual exposure time (subtract 2.7 ms), MK, 23 09 05
;-

FUNCTION osiris_exposuretime, header, image, quality_map, sigma_map, PARAMETER_FILELIST=PARAMETER_FILELIST, _EXTRA=_EXTRA
VERSION='V1.3'

    if keyword_set(VERBOSE) then print, 'osiris_exposuretime running'

    ; Verify the the correction has not already been performed
    flag = p_value(header, 'LEVEL2_PIPELINE.EXPOSURETIME_CORRECTION_FLAG', /HISTORY)

    if flag eq 'TRUE' then begin
       if keyword_set(VERBOSE) then print, 'osiris_exposuretime skipped - Already performed'
       return, 1
    endif


    ; do the calibration
    proc_message = 'Fow just taking exposure time and hardware binning into account'

    time = p_value(header, 'SR_ACQUIRE_OPTIONS.EXPOSURE_DURATION')
    bin = p_value(header, 'SR_ACQUIRE_OPTIONS.HARDWARE_BINNING_ID')


	; Extract the real exposure time from the shutter motion
	;
	time = osiris_shutter(header, VERSION=shm_version)

	if time[0] lt 0 then begin
		printf, -2, 'WARNING: Shutter calibration failed - falling back to fixed offset'
    	; time = time - 0.0027		; Old implementation
    	osiris_model, header, camera, model
    	if camera eq 'NAC' then time -= 0.0024 else time -= 0.0033
    	p_setvalue, header, 'LEVEL2_PIPELINE.EXPTIME_CALIB_METHOD', 'FIXED_OFFSET', /LABEL, /HISTORY
	endif else  p_setvalue, header, 'LEVEL2_PIPELINE.EXPTIME_CALIB_METHOD', 'SHM_PULSE' + shm_version, /LABEL, /HISTORY


    ; correct time for the hardware binning factor
    if bin eq '2x2' then time *= 4.0
    if bin eq '4x4' then time *= 16.0
    if bin eq '8x8' then time *= 64.0

	; Save the mean exposure time to the histore header
	p_setvalue, header, 'LEVEL2_PIPELINE.MEAN_EFFECTIVE_EXPOSURETIME', mean(time), /GROUP, /HISTORY

    ; convert to DN/s
    image = image / time

    ; ToDo: Add correction for shutter motion


    ; Change the processing flag
    p_setvalue, header, 'LEVEL2_PIPELINE.EXPOSURETIME_CORRECTION_FLAG', 'TRUE', /LABEL, /NO_QUOTES, /HISTORY
    p_setvalue, header, 'LEVEL2_PIPELINE.EXPOSURETIME_CORRECTION_VERSION', VERSION, /GROUP, /HISTORY


    ; finish
    if keyword_set(VERBOSE) then print, 'osiris_exposuretime - Done'


    return, 1
END