; Copyright (c) 2004-2010, Stubbe F. Hviid
;   All rights reserved
;+
; NAME:
;   FUNCTION osiris_flatfield, header, image
;
; PURPOSE:
;   FLatfield correct an Osiris image
;
; CATEGORY:
;   Osiris CCD Calibration
;
; CALLING SEQUENCE:
;  Result = osiris_flatfield(header, image, quality_map, sigma_map)
;
; INPUTS:
;   header:   		The PDS image header of the image to be corrected
;   image:      	The image data of the image to be corrected
;   quality_map:	Existing map of quality estimate for each pixel to be modified
;   sigma_map:		Existing map of estimaped relative error for each pixel (in %) to be modified
;
; KEYWORD PARAMETERS:
;   PARAMETER_FILELIST:	all external datafile names neede for this correction step
;				        will be added to this keywor parameter in a string array
;
; OUTPUTS:
;   header:   		The PDS image header of the image after correction
;   image:      	The image data of the image after correction
;   quality_map:	output map of quality estimate for each pixel
;   sigma_map:		output map of estimaped relative error for each pixel (in %)
;
; RETURNS:
;   0 on failure   1 on success
;
; RESTRICTIONS:
;   function requires that the paths defined in get_osiris_environment.pro has been set to correctly
;   point to the root directory of the external calibration files
;
;   The function depends on an external data file <camera>_<unit>_FLAT-<filter1>-<filetr2>_<version>_<timestamp>.img
;   to exist in the Flatfields folder of the database tree. This file must be a PDS image file containing
;   the flatfield for the <filter1>, <filter2> combination
;
; EXAMPLE:
;   p_read, pickfile(), h, d             ; read in raw PDS image
;   osiris_flatfield, h, d
;
; MODIFICATION HISTORY:
;   Written by:  Stubbe F. Hviid   28/6-2005
;                Sfh 02102006 - Switched to using osiris_model for detecting the camera and hardware model
;-

FUNCTION osiris_flatfield, header, image, quality_map, sigma_map, PARAMETER_FILELIST=PARAMETER_FILELIST, _EXTRA=_EXTRA
VERSION='V1.1'

    if keyword_set(VERBOSE) then print, 'osiris_flatfield running'

    ; Verify the the correction has not already been performed
    flag = p_value(header, 'LEVEL2_PIPELINE.FLATFIELD_CORRECTION_FLAG', /HISTORY)

    if flag eq 'TRUE' then begin
       if keyword_set(VERBOSE) then print, 'osiris_flatfield skipped - Already performed'
       return, 1
    endif

    ; do the calibration

	; extract some information from the image header
	osiris_model, header, camera, model

	fnumber = strtrim(p_value(header, 'SR_MECHANISM_STATUS.FILTER_NUMBER'), 2)
	bin = p_value(header, 'SR_ACQUIRE_OPTIONS.HARDWARE_BINNING_ID')
	swbin = (p_value(header, 'SR_COMPRESSION.PIXEL_AVERAGING_WIDTH'))[0]

	; begin - archive compatibility step to handle change in FILTER_NUMBER definition
	if size(fnumber, /DIM) eq 2 then fnumber = fnumber[0] + fnumber[1]
	; end   - archive compatibility step to handle change in FILTER_NUMBER definition

	; load required flatfield from database
	flatfield_file = p_resolve_database_file('FLATFIELDS', camera, model, 'FLAT' + '-' + fnumber, '', 'IMG')

	if flatfield_file eq '' then begin
		printf, -2, 'ERROR: Cannot find required flatfield for filters: (' + fnumber + ') - osiris_flatfield'
		return, 0
	endif

	df = p_parse_file_path(flatfield_file)
	proc_message = 'Flatfielding using: ' + df.filename

	; load the flatfield
	p_read, flatfield_file, fh, df

	; add the flatfield file to the PARAMETER_FILELIST
	if n_elements(PARAMETER_FILELIST) eq 0 then PARAMETER_FILELIST = [flatfield_file] else PARAMETER_FILELIST = [PARAMETER_FILELIST, flatfield_file]


	; extract the correct subframe
	xoff = p_value(header, 'IMAGE.FIRST_LINE_SAMPLE') - 1
	yoff = p_value(header, 'IMAGE.FIRST_LINE') - 1


	; consistency check!
	if xoff lt 0 OR yoff lt 0 then begin
		printf, -2, 'ERROR: Illegal subframe'
		return, 0
	endif
	; end consistency check!

	bf = 1
	if bin eq '2x2' then bf = 2
	if bin eq '4x4' then bf = 4
	if bin eq '8x8' then bf = 8

	if swbin gt 1 then bf = bf * swbin

	dim = size(image, /DIM)

	; determine size of original image
	fw = dim[0] * bf
	fh = dim[1] * bf

	; extract a subfram out of the flatfield of the same size
	sub_flat = df[xoff:xoff+fw-1, yoff:yoff+fh-1]

	; average the flatfield down to the binned size
	if bf gt 1 then sub_flat = rebin(sub_flat, dim[0], dim[1])

	; finally flatfield the image
	image = image / sub_flat


    ; Change the processing flag
    p_setvalue, header, 'LEVEL2_PIPELINE.FLATFIELD_CORRECTION_FLAG', 'TRUE', /LABEL, /NO_QUOTES, /HISTORY
    p_setvalue, header, 'LEVEL2_PIPELINE.FLATFIELD_CORRECTION_VERSION', VERSION, /GROUP, /HISTORY


    ; finish
    if keyword_set(VERBOSE) then print, 'osiris_flatfield - Done'

    return, 1
END