; Copyright (c) 2005, Stubbe F. Hviid
;   All rights reserved
;+
; NAME:
;   fw_save_tagfile
;
; PURPOSE:
;   Save an IDL structure to a tag=value sture file. The input structure must be a simple structure
;   where no members are of type structure, pointer of object
;   scalar arrays and string arrays are supported
;
; CATEGORY:
;   file io
;
; CALLING SEQUENCE:
;   PROCEDURE fw_load_tagfile, file, structure
;
;
; INPUTS:
;   file:        filename of file to be written
;   structure:   the data structure to be written
;
; OPTIONAL INPUTS:
;   None
;
; KEYWORD PARAMETERS:
;   QUIET:   If set no progress messges will be printed only error messages
;
; OUTPUTS:
;   None
;
; OPTIONAL OUTPUTS:
;   None
;
; COMMON BLOCKS:
;   None
;
; SIDE EFFECTS:
;   A write failure will generate a stop
;
; RESTRICTIONS:
;   None
;
; EXAMPLE:
;  x = {Hello: "World", Tag1: 10, Tag2: [1,2,3,4], Tag3: [1.0, 2.0, 3.0, 4.0], Tag4: ['Hello', 'World']}
;  fw_save_tagfile, 'test.dat', x
;
; MODIFICATION HISTORY:
;   Written by:  Stubbe F. Hviid  June 2005
;-

PRO fw_save_tagfile, file, structure, QUIET=QUIET

    ; define error handling
    on_ioerror, error

    ; Tell the world that we are reading <file>
    if NOT keyword_set(QUIET) then print, 'Saving tagfile: ' + file

    ; Check input structure for compatibility
    names = TAG_NAMES(structure)

    if size(structure, /TYPE) ne 8 then begin
       print, 'ERROR: Input not containing a structure'
       stop
       return
    endif

    for i=0, N_TAGS(structure)-1 do begin
       t = size(structure.(i), /TYPE)
       if (t eq 8) OR (t eq 10) OR (t eq 11)then begin
         print, 'ERROR: Incompatible structure member data type in member: ' + names[i]
         stop
         return
       endif

       d = size(structure.(i), /N_DIM)
       if d gt 1 then begin
         print, 'Member with more than 0 or 1 dimensions found in tag: ' + names[i]
         stop
         return
       endif

       if n_elements(structure.(i)) gt 40 then begin
         print, 'vector with more than 40 elements found in: ' + names[i]
         stop
         return
       endif
    endfor


    ; try to open the file
    openw, unit, file, /GET_LUN
    for i=0, N_TAGS(structure)-1 do begin
       tag = names[i]

       if size(structure.(i), /N_DIM) eq 0 then begin
         ; scalar member
         if size(structure.(i), /TYPE) eq 7 then begin
           ; string
           printf, unit, names[i] + '="' + strtrim(structure.(i), 2) + '"'
        endif else begin
         ; number
         printf, unit, names[i] + '=' + strtrim(structure.(i), 2)
        endelse
       endif else begin
         ; vector member
        value = ''
        if size(structure.(i), /TYPE) eq 7 then begin
          for j=0, n_elements(structure.(i))-1 do begin
             value = value + ', ' + '"' + strtrim(structure.(i)[j], 2) + '"'
          endfor
        endif else begin
          for j=0, n_elements(structure.(i))-1 do begin
             value = value + ', ' + strtrim(structure.(i)[j], 2)
          endfor
        endelse
        value = strmid(value, 2)   ; skip first heading ', '
        printf, unit, names[i] + '=[' + value + ']'
       endelse
    endfor
    saved_count = 0

    ; close input file and unit
    close, unit
    free_lun, unit

    ;if NOT keyword_set(QUIET) then print, 'Saved ' + saved_count + ' tags to file'


    ; finish normal processing
    return

    ; Error handling
    error:
    print, 'Error saving tag file: ' + file
    stop
END